#!/usr/bin/perl

######################################################################
package Cognition::uF;
######################################################################

use DateTime;
use DateTime::Format::ISO8601;
use DateTime::Format::Natural;
use DateTime::Format::Strptime;
use XML::LibXML qw(:all);
use strict;

BEGIN
{
	our $VERSION = 0.1;
}


sub MAX_INCLUDE_ITERATIONS { 2; }


sub ufp_abbr
# Implements the microformats ABBR pattern.
{
	my $node = shift;
	my $page = shift;
	my $elem = shift || 'abbr';
	my $attr = shift || 'title';
	my $rv = 0;
	
	foreach my $abbr ($node->getElementsByTagName('*'))
	{
		# Skip pattern if no title attribute found.
		my $title = $abbr->getAttribute($attr) || next;

		# ABBR element (or other element specified as third parameter) should
		# *always* be expanded. Other elements should only be expanded if the
		# title attribute (or other attribute, fourth parameter) includes the
		# string 'data:'.
		next
			unless ($abbr->localname eq $elem 
			|| $abbr->getAttribute($attr) =~ /data\:/);
		
		# Implement Andy Mabbett's suggested "data:" prefix
		if ($title =~ / [\(\[] data\: (.*) [\)\]] /x
		||  $title =~ / data\: (.*) $ /x )
		{
			$title = $1;
		}
		
		# Remove child nodes
		foreach my $kid ($abbr->getChildNodes)
		{
			$abbr->removeChild($kid);
		}
		
		# Insert contents of title attribute
		$abbr->appendChild(XML::LibXML::Text->new($title)) && $rv++;
	}
	
	# Return number of replacements made.
	return $rv;
	
} # /sub ufp_abbr

sub ufp_table
{
	my $node = shift;
	my $page = shift;
	
	# Add node itself to list!
	my @elements = $node->getElementsByTagName('td');
	if ($node->tagName =~ /^t[dh]$/i)
		{ unshift @elements, $node; }
		
	foreach my $tag (@elements)
	{
		next unless (length $tag->getAttribute('headers'));
		
		my $headers = $tag->getAttribute('headers');
		$headers =~ s/(^\s+|\s+$)//g;
		$headers =~ s/\s+/ /g;
		my @headers = split / /, $headers;
		
		foreach my $H (@headers)
		{
			my $Htag = Cognition::HTMLParser::Utils::searchID($H, $page->{DOM});
			next unless ($Htag);
			next unless ($Htag->tagName =~ /^t[dh]$/i);
			
			if (length $Htag->getAttribute('axis'))
			{
				$tag->setAttribute('class',
					$Htag->getAttribute('axis').' '.$tag->getAttribute('class'));
			}
			
			my $new = $page->{DOM}->createElement('div');
			$new->setAttribute('class', $Htag->getAttribute('class'));
			foreach my $kid ($Htag->childNodes)
			{
				my $x = $kid->cloneNode(1);
				if ($kid->nodeType==XML_ELEMENT_NODE)
				{
					my $r = $new->appendChild($x);
				}
			}
			$tag->appendChild($new);
		}
		
		$tag->setAttribute('headers', '');
	}
}

sub ufp_include
# Implements the standard microformats include pattern.
{
	my $node = shift;
	my $page = shift;
	my $class = shift || 'include';
	my $rv = 0;
	
	# For each link...
	foreach my $link ($node->getElementsByTagName('a'))
	{	
		# Skip pattern if no class attribute found.
		my $classList = $link->getAttribute('class') || next;
		
		# We've found a use of the include pattern
		if ($classList =~ / (^|\s) $class (\s|$) /x)
		{
			my $href = $link->getAttribute('href');
			my $id = undef;
			if ($href =~ /^\#(.*)$/)
			{
				$id = $1;
			}
			else
			{
				next;
			}
			
			my $replacement = Cognition::HTMLParser::Utils::searchID($id, $page->{DOM})->cloneNode(1) || next;
			$link->getParentNode->replaceChild($replacement, $link) && $rv++;
		}
	}
	
	# Return number of replacements made.
	return $rv;
	
} # /sub ufp_include


sub ufp_include2
# Implements the alternative microformats include pattern.
{
	my $node = shift;
	my $page = shift;
	my $classpfx = shift || '#';
	my $rv = 0;
	
	# Add node itself to list!
	my @elements = $node->getElementsByTagName('*');
	unshift @elements, $node;
	
	# For each element...
	foreach my $elem (@elements)
	{
		# Skip pattern if no class attribute found.
		my $classList = $elem->getAttribute('class');
		next unless ($classList =~ / $classpfx /x);
		
		my $atEnd = 0;
		
		$classList =~ s/(^\s|\s$)//g;
		$classList =~ s/\s+/ /g;
		my @classes = split / /, $classList;
		my @newClassList = ();
		
		foreach my $c (@classes)
		{
			if (substr($c,0,1) ne $classpfx && length($c)>1)
			{
				push @newClassList, $c;
				$atEnd = 1;
				next;
			}
			
			my $id = $c; $id =~ s/^\#//x;
			my $replacement = Cognition::HTMLParser::Utils::searchID($id, $page->{DOM}) || next;

			$replacement = $replacement->cloneNode(1);
			if ($atEnd)
			{
				$elem->appendChild($replacement) && $rv++;
			}
			else
			{
				$elem->insertBefore($replacement, $elem->getFirstChild) && $rv++;
			}
		}
		
		$elem->setAttribute('class', join(' ', @newClassList));
	}
	
	# Return number of replacements made.
	return $rv;
	
} # /sub ufp_include2


sub datetime
# Very tolerant DateTime parsing. Microformats are supposed to always use W3CDTF,
# but we'll be lenient and do our best to parse other ISO 8601 formats, and if
# that fails, even try to parse natural language.
{
	my $string = shift;
	my $tz     = shift||'UTC';
	my $f;
	my $dt;

	$f = DateTime::Format::ISO8601->new;
	eval { $dt = $f->parse_datetime($string); };

	unless ($dt)
	{
		$f = DateTime::Format::Natural->new(
			lang          => 'en',        # Should read this from source input
			prefer_future => 1,
			daytime       => {
										morning    =>  9,
										afternoon  => 13,
										evening    => 20
								  },
			time_zone     => $tz
		);
		$dt = $f->parse_datetime($string);
	}
	$dt = DateTime->now unless ($dt);
	
	$dt->set_formatter(DateTime::Format::Strptime->new(pattern=>'%FT%T.%6N%z'));
	
	return $dt;
}


1; #/package Cognition::HTMLParser::Microformats




