#!/usr/bin/perl

######################################################################
package Cognition::uF::geo;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchClass('geo', $within);
	my @rv;
	
	foreach my $geo (@nodes)
	{
		my $geo_parsed = parse($page, $geo);
		push @rv, $geo_parsed;
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	unless ((defined $this->{body} && $this->{body} !~ /^\s*earth\s*$/i)
	||      (defined $this->{'reference-frame'} && $this->{'reference-frame'} !~ /wgs.*84/i ))
	{
		if (length $this->{altitude})
		{
			push @rv, sprintf('geo:%f,%f,%f', 
				$this->{latitude},
				$this->{longitude},
				$this->{altitude});
		}
		else
		{
			push @rv, sprintf('geo:%f,%f', 
				$this->{latitude},
				$this->{longitude});
		}
	}
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->setDom($this->{_dom});
	$subject->addObject($this);
	
	if ( (defined $this->{body} && $this->{body} !~ /^\s*earth\s*$/i)
	||   (defined $this->{'reference-frame'} && $this->{'reference-frame'} !~ /wgs.*84/i ) )
	{
		my $comment   = Cognition::RDFModel::Value->new('Non-WGS84 geopositioning.');
		$subject->addVal('http://www.w3.org/2000/01/rdf-schema#comment', $comment);
		
		foreach my $v (qw(altitude longitude latitude reference-frame body))
		{
			if (length $this->{$v})
			{
				my $val = Cognition::RDFModel::Value->new($this->{$v});
				$subject->addVal('http://microformats.org/wiki/geo-extension-nonWGS84#'.$v, $val);
			}
		}
	}
	else
	{
		$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
			Cognition::RDFModel::Value->new("http://www.w3.org/2003/01/geo/wgs84_pos#Point", 1));

		my $latitude  = Cognition::RDFModel::Value->new($this->{latitude});
		my $longitude = Cognition::RDFModel::Value->new($this->{longitude});
		
		$subject->addVal('http://www.w3.org/2003/01/geo/wgs84_pos#latitude', $latitude);
		$subject->addVal('http://www.w3.org/2003/01/geo/wgs84_pos#longitude', $longitude);
		
		if (length $this->{altitude})
		{
			my $altitude = Cognition::RDFModel::Value->new($this->{altitude});
			$subject->addVal('http://www.w3.org/2003/01/geo/wgs84_pos#longitude', $altitude);
		}
	}
	
	return $subject;
}


sub metadata_ns
{
	my $this = shift;
	return { 'geo'=> { 
		nsurl => 'http://www.w3.org/2003/01/geo/wgs84_pos#',
		title => 'Basic Geo (WGS84 lat/long) Vocabulary' 
	},
	'ungeo' => {
		nsurl => 'http://microformats.org/wiki/geo-extension-nonWGS84#',
		title => 'Non-WGS84 Geopositioning Vocabulary'
	}};
}


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $geo_node = $rv->{'_dom'}->cloneNode(1);

	my $id = $geo_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < Cognition::uF::MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($geo_node, $page) 
		              + Cognition::uF::ufp_include2($geo_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($geo_node, $page);
	Cognition::uF::ufp_table($geo_node, $page);
	
	# Simple attributes - singles
	my @simples = qw(longitude latitude body reference-frame altitude);	
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $geo_node);
		my $value_node  = $value_nodes[0];
		next unless ($value_node);
		my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node);
		$rv->{$simple} = $value;
	}
	
	if (!length($rv->{longitude}) || !length($rv->{latitude}))
	{
		my $str = $geo_node->toString;
		if ($geo_node->localname eq 'img' || $geo_node->localname eq 'area')
			{ $str = $geo_node->getAttribute('alt'); }
			
		$str =~ s/\,/\./g;
		if ($str =~ / \s* (\-?[0-9\.]+) \s* \; \s* (\-?[0-9\.]+) \s* /x)
		{
			$rv->{latitude}  = $1;
			$rv->{longitude} = $2;
		}
	}
	
	bless $rv;
	return $rv;
	
} #/sub parse


1; #/package Cognition::uF::geo




