#!/usr/bin/perl

######################################################################
package Cognition::uF::hEvent;
######################################################################

use XML::LibXML qw(:all);
use strict;
use CGI::Util;
use Digest::SHA1 qw(sha1_hex);

BEGIN
{
	our $VERSION = 0.1;
	our @ISA = qw(Cognition::uF);
}


sub MAX_INCLUDE_ITERATIONS { 6; }


sub parse_all
{
	my $page   = shift;
	my $within = shift || $page->{DOM};
	my @nodes = Cognition::HTMLParser::Utils::searchClass('vevent', $within);
	my @rv;
	
	foreach my $ev (@nodes)
	{
		push @rv, parse($page, $ev);
	}
	
	return @rv;
} #/sub parse_all


sub uri
{
	my $this = shift;
	my $all  = shift;
	my @rv;
	
	if (length $this->{uid})
	{
		push @rv, $this->{uid};
	}
	if (length $this->{_id})
	{
		push @rv, $this->{_page}->abs_url.'#'.$this->{_id};
	}
	if (length $this->{summary})
	{
		push @rv, 'urn:x-vevent:' . CGI::Util::escape($this->{summary});
	}
	push @rv, Cognition::HTMLParser::Utils::XPointer($this->{_dom}, $this->{_page}->abs_url);
	
	return $rv[0] unless (defined $all);
	while ($all) { shift @rv; $all--; }
	return @rv;
} #/sub uri


sub rdf_subject
{
	my $this    = shift;
	my $ns      = 'urn:ietf:rfc:2445#';
	my $CLASS   = shift;

	# Create RDF subject
	my $subject = Cognition::RDFModel::Subject->new($this->uri);
	$subject->addObject($this);
	$subject->setDom($this->{_dom});
	
	if ($CLASS =~ /htodo$/i)
	{
		$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
			Cognition::RDFModel::Value->new($ns.'vtodo', 1));
	}
	else
	{
		$subject->addVal('http://www.w3.org/1999/02/22-rdf-syntax-ns#type',
			Cognition::RDFModel::Value->new($ns.'vevent', 1));
	}
	
	# Patterns for simple data.
	my @_SINGLES  = qw(class description dtend dtstart duration status transp
		summary last-modified comment priority created sequence due
		percent-complete completed);
	my @_SINGLESR = qw(uid url recurrance-id);
	my @_MULTIS   = qw(rdate rrule resources exdate exrule);
	my @_MULTISR  = qw(related-to);
	my @_OBJECTS  = qw(contact attendee organiser location category attach);
	
	# Properties which exist as singles.
	foreach my $property (@_SINGLES)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property});
		$subject->addVal($ns.$property, $val);
	}

	# Resource-type properties which exist as singles.
	foreach my $property (@_SINGLESR)
	{
		next unless (length $this->{$property});
		my $val = Cognition::RDFModel::Value->new($this->{$property}, 1);
		$subject->addVal($ns.$property, $val);
	}
		
	# Properties which exist as multiple values.
	foreach my $property (@_MULTIS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V);
			$subject->addVal($ns.$property, $val);
		}
	}
	
	# Resource-type properties which exist as multiple values.
	foreach my $property (@_MULTISR)
	{
		foreach my $V (@{ $this->{$property} })
		{
			next unless (length $V);
			my $val = Cognition::RDFModel::Value->new($V, 1);
			$subject->addVal($ns.$property, $val);
		}
	}

	# Embedded objects which may be represented as resources
	foreach my $property (@_OBJECTS)
	{
		foreach my $V (@{ $this->{$property} })
		{
			my $val = Cognition::RDFModel::Value->new($V->uri, 1);
			$subject->addVal($ns.$property, $val);
		}
	}
	
	return $subject;
	
} #/sub rdf_subject


sub metadata_ns
{
	my $this = shift;
	return { 'ICAL' => { nsurl=>'urn:ietf:rfc:2445#',title=>'iCalendar' } };
} #/sub metadata_ns


sub parse
{
	my $page = shift;
	my $rv = { '_dom'=>shift };
	my $CLASS = shift;
	$CLASS = (length $CLASS) ? $CLASS : 'Cognition::uF::hEvent';
	my $event_node = $rv->{'_dom'}->cloneNode(1);
	
	my $id = $event_node->getAttribute('id');
	$rv->{_id} = $id if (length $id);
	$rv->{_page} = $page;
	
	# Expand microformat patterns.
	my $incl_iterations = 0;
	my $replacements = 1;
	while (($incl_iterations < MAX_INCLUDE_ITERATIONS) && $replacements)
	{
		$replacements = Cognition::uF::ufp_include($event_node, $page) 
		              + Cognition::uF::ufp_include2($event_node, $page);
		$incl_iterations++;
	}
	Cognition::uF::ufp_abbr($event_node, $page);
	Cognition::uF::ufp_table($event_node, $page);

	# Pull out nested hcards to prevent them being parsed as within this event.
	# Check to see if they have a 'useful' class, and if so, parse them in.
	my @nested = Cognition::HTMLParser::Utils::searchClass('vcard', $event_node);
	foreach my $h (@nested)
	{
		if ($h->getAttribute('class') =~ / (^|\s) organizer (\s|$) /x)
		{
			push @{ $rv->{organizer} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		if ($h->getAttribute('class') =~ / (^|\s) contact (\s|$) /x)
		{
			push @{ $rv->{contact} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		if ($h->getAttribute('class') =~ / (^|\s) attendee (\s|$) /x)
		{
			push @{ $rv->{attendee} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		if ($h->getAttribute('class') =~ / (^|\s) location (\s|$) /x)
		{
			push @{ $rv->{location} }, Cognition::uF::hCard::parse($page, $h);
		}
		
		$h->getParentNode->removeChild($h);
	}
	
	# Ditto for addresses
	my @nested = Cognition::HTMLParser::Utils::searchClass('adr', $event_node);
	foreach my $h (@nested)
	{
		if ($h->getAttribute('class') =~ / (^|\s) location (\s|$) /x)
		{
			push @{ $rv->{location} }, Cognition::uF::adr::parse($page, $h);
		}
		
		$h->getParentNode->removeChild($h);
	}

	# Ditto for geo
	my @nested = Cognition::HTMLParser::Utils::searchClass('geo', $event_node);
	foreach my $h (@nested)
	{
		if ($h->getAttribute('class') =~ / (^|\s) location (\s|$) /x)
		{
			push @{ $rv->{location} }, Cognition::uF::geo::parse($page, $h);
		}
		else
		{
			push @{ $rv->{geo} }, Cognition::uF::geo::parse($page, $h);
		}
		
		$h->getParentNode->removeChild($h);
	}

	# Nested VALARMs.
	my @nested = Cognition::HTMLParser::Utils::searchClass('valarm', $event_node);
	foreach my $h (@nested)
	{
		push @{ $rv->{valarm} }, Cognition::uF::hAlarm::parse($page, $h);
		$h->getParentNode->removeChild($h);
	}

	# Categories
	my @tags = Cognition::uF::RelTag::parse_all($page, $event_node);
	$rv->{category} = \@tags unless (!@tags);	
	
	# Enclosures
	my @encs = Cognition::uF::RelEnclosure::parse_all($page, $event_node);
	$rv->{attach} = \@encs unless (!@encs);
	
	# Simple attributes - singles
	my @simples = qw(class description dtend dtstart duration status transp
		summary uid last-modified comment recurrance-id priority created
		sequence due percent-complete completed);
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $event_node);
		my $value_node  = $value_nodes[0];
		next unless ($value_node);
		
		my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node);

		if ($simple =~ /^(uid|recurrance.id)$/)
		{
			if ($simple eq 'uid' && length $value_node->getAttribute('id'))
				{ $value = $page->abs_url . '#' . $value_node->getAttribute('id'); }
			elsif ($simple eq 'uid' && ($value_node->localname eq 'a' || $value_node->localname eq 'area') && length $value_node->getAttribute('name'))
				{ $value = $page->abs_url . '#' . $value_node->getAttribute('name'); }
			elsif ($value_node->localname eq 'a' || $value_node->localname eq 'area')
				{ $value = $page->abs_url($value_node->getAttribute('href')); }
			elsif ($value_node->localname eq 'img')
				{ $value = $page->abs_url($value_node->getAttribute('src')); }
			elsif ($value_node->localname eq 'object')
				{ $value = $page->abs_url($value_node->getAttribute('data')); }
		}

		if ($simple =~ /^(last.modified|dtstart|dtend|created|due|completed)$/ && length $value)
		{
			$value = Cognition::uF::datetime($value);
			$value->subtract(seconds=>1)
				if ($value_node->getAttribute('class') =~ /beforethis/i && $simple =~ /(dtend|due)/);
		}
			
		$rv->{$simple} = $value;
	}	

	# Simple attributes - possible duplicates
	my @simples = qw(resources related-to exdate exrule rdate rrule organiser
		contact attendee location url);
	foreach my $simple (@simples)
	{
		my @value_nodes = Cognition::HTMLParser::Utils::searchClass($simple, $event_node);
		foreach my $value_node (@value_nodes)
		{
			next unless ($value_node);
			
			my $value = Cognition::HTMLParser::Utils::STRINGIFY($value_node);
			
			if ($simple =~ /^(related.to|url)$/)
			{
				if ($value_node->localname eq 'a' || $value_node->localname eq 'area')
					{ $value = $page->abs_url($value_node->getAttribute('href')); }
				elsif ($value_node->localname eq 'img')
					{ $value = $page->abs_url($value_node->getAttribute('src')); }
				elsif ($value_node->localname eq 'object')
					{ $value = $page->abs_url($value_node->getAttribute('data')); }
			}

			if ($simple =~ /^(rdate|exdate)$/ && length $value)
			{
				$value = Cognition::uF::datetime($value);
				$value->subtract(seconds=>1)
					if ($value_node->getAttribute('class') =~ /beforethis/i);
			}

			if ($simple =~ /^(rrule|exrule)$/)
				{ $value = recurrance($value_node); }
			
			push @{$rv->{$simple}}, $value;
		}
	}	
	bless $rv, $CLASS;
	return $rv;
	
} #/sub parse


sub recurrance
{
	my $node = shift;
	my %props = {};
	
	my @freq_nodes = Cognition::HTMLParser::Utils::searchClass('freq', $node);
	unless (defined $freq_nodes[0])
	{
		if (lc($node->tagName) eq 'abbr' && length $node->getAttribute('title'))
			{ return $node->getAttribute('title'); }
		else
			{ return Cognition::HTMLParser::Utils::STRINGIFY($node); }
	}

	$props{freq} = uc(Cognition::HTMLParser::Utils::STRINGIFY($freq_nodes[0]));
	
	foreach my $n ($node->getElementsByTagName('*'))
	{
		if ($n->getAttribute('class') =~ /\b (until|count) \b/x)
		{
			unless (defined $props{'until'} || defined $props{'count'})
			{
				$props{$1} = Cognition::HTMLParser::Utils::STRINGIFY($n);
				$props{$1} = Cognition::uF::datetime($props{$1}) if ($1 eq 'until');
			}
		}
		
		if ($n->getAttribute('class') =~ /\b (bysecond | byminute | byhour |
			bymonthday | byyearday | byweekno | bymonth | bysetpos) \b/x)
		{
			my $v = Cognition::HTMLParser::Utils::STRINGIFY($n);
			if (length $props{$1})
			{
				$props{$1} .= ",$v";
			}
			else
			{
				$props{$1} = "$v";
			}
		}

		elsif ($n->getAttribute('class') =~ /\b (byday | wkst) \b/x)
		{
			my $v = substr(uc(Cognition::HTMLParser::Utils::STRINGIFY($n)), 0, 2);
			if (length $props{$1})
			{
				$props{$1} .= ",$v";
			}
			else
			{
				$props{$1} = "$v";
			}
		}
		
		if ($n->getAttribute('class') =~ /\b interval \b/x)
		{
			my $v = Cognition::HTMLParser::Utils::STRINGIFY($n);
			$props{'interval'} = $v;			
		}
	}
	
	my $rv = '';
	foreach my $k (keys %props)
	{
		$rv .= uc($k).'='.$props{$k}.';'
			unless ($k =~ /^HASH/); # huh?
	}
	$rv =~ s/\;$//;
	
	return $rv;
}


1; #/package Cognition::uF::hEvent




